/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2008-2021, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#ifndef headerfile_winix_models_item
#define headerfile_winix_models_item

#include <string>
#include "model.h"
#include "models/itemcontent.h"


namespace Winix
{

// IMPROVEME rename me and move to a better place
class CalcItemsHelper : public morm::Model
{
public:

	long size;

	void map_fields()
	{
		field(L"size", size);
	}

};


class Item : public morm::Model
{
public:


	enum Type
	{
		dir  = 0,
		file = 1,
		symlink = 2,

		none = 1000
	};



	/*
	 *
	 */
	long id;

	/*
	 *
	 */
	long parent_id;

	/*
	 * may change to int?
	 */
	Type type;

	/*
	 *
	 */
	std::wstring url;

	/*
	 *
	 */
	std::wstring subject;

	/*
	 *
	 */
	std::wstring html_template;

	/*
	 * sort index used when displaying a group of items
	 */
	int sort_index;

	/*
	 * may not needed?
	 * the id is in the ItemContent object
	 */
	long content_id;

	/*
	 *
	 */
	ItemContent item_content;




	Item();

	void map_fields();
	void table_name(PT::TextStream & stream);
	void before_insert();
	void after_insert();

	/*
	 * what about clear() from Model?
	 */
	void Clear();


	bool prepare_url();


	/*
	 * CHECKME
	 * make sure update(...) methods from Model are not available
	 */
	bool update(morm::ModelData * model_data, bool url_was_changed, bool update_whole_tree = true);
	bool update(morm::ModelData & model_data, bool url_was_changed, bool update_whole_tree = true);
	bool update(bool url_was_changed, bool update_whole_tree = true);
	// IMPROVEME make the same for save() methods


	/*
	 * IMPROVEME
	 * when removing we should check whether the item_content.references is zero and them remove it
	 * in other cases only decrement item_content.references
	 *
	 */


	/*
	 * temporary
	 */
	static void do_migration(morm::ModelConnector * model_connector);

protected:

	bool url_was_prepared_correctly;


	long calc_items_by_url(long parent_id, const std::wstring & url);


};


} // namespace Winix



#endif
